Imports System.Text
Imports System.IO
Imports System.Collections

Namespace System

   Public Class Extremum(Of T As IComparable)
      Private _MinVal As T
      Private _MaxVal As T
      Private _IsNew As Boolean
      Public Sub New(ParamArray Values() As T)
         Reset(Values)
      End Sub
      Public Function Reset(ParamArray Values() As T) As Extremum(Of T)
         _IsNew = True
         Return CheckNew(Values)
      End Function
      Public Function CheckNew(ParamArray Values() As T) As Extremum(Of T)
         For Each Val As T In Values
            If _IsNew Then
               _MinVal = Val
               _MaxVal = Val
               _IsNew = False
            ElseIf Val.CompareTo(_MinVal) < 0 Then
               _MinVal = Val
            ElseIf Val.CompareTo(_MaxVal) > 0 Then
               _MaxVal = Val
            End If
         Next
         Return Me
      End Function
      Public ReadOnly Property HasValue() As Boolean
         Get
            Return Not _IsNew
         End Get
      End Property
      Public ReadOnly Property MinVal() As T
         Get
            If _IsNew Then
               Throw New ArgumentOutOfRangeException("_MinVal", "didn't check any value")
            End If
            Return _MinVal
         End Get
      End Property
      Public ReadOnly Property MaxVal() As T
         Get
            If _IsNew Then
               Throw New ArgumentOutOfRangeException("_MaxVal", "didn't check any value")
            End If
            Return _MaxVal
         End Get
      End Property
   End Class 'Extremum(Of T As IComparable)

   Public Class clsLevel
      'erleichtert stack-miges temporres disablen von Eigenschaften.
      'z.B. Befllung eines Treenodes im Before_Expand-Event: Mu natrlich TV.Suspend/Resume-Layout
      '! einsetzen. Nun will ich von woanders aber eine Reihe von Nodes expanden. Das gbe dann ein
      '! ziemliches Suspend/Resume - Geklapper.
      'Beispiel:
      'Private _SuspendLevel As New clsLevel
      'If _SuspendLevel.Increment=1 Then Treeview.SuspendLayout()
      '...
      'If _SuspendLevel.Decrement=0 Then Treeview.ResumeLayout()
      Private _Value As Integer
      Public Sub New(FirstLevel As Integer)
         _Value = FirstLevel
      End Sub
      Public ReadOnly Property Level() As Integer
         Get
            Return _Value
         End Get
      End Property
      Public Function Increment() As Integer
         _Value += 1
         Return _Value
      End Function
      Public Function Decrement() As Integer
         _Value -= 1
         If _Value < 0 Then
            Throw New ArgumentOutOfRangeException( _
                "Value", Level, "Ein Level sollte nicht unter 0 gehen")
         End If
         Return _Value
      End Function
   End Class 'clsLevel

   Public Class clsChangeChecker(Of T)
      Private Obj As Object = Nothing
      Public Sub New()
      End Sub
      Public Sub New(InitVal As Object)
         Obj = InitVal
      End Sub
      Default Public ReadOnly Property IsChange(ToCheck As Object) As Boolean
         Get
            If Object.Equals(Obj, ToCheck) Then Return False
            Obj = ToCheck
            Return True
         End Get
      End Property
      Public ReadOnly Property Value() As T
         Get
            Return DirectCast(Obj, T)
         End Get
      End Property
   End Class 'clsChangeChecker

   Public Class TableDebugger
      'layoutet Debugging-zeilen mit vielen Werten als Tabelle
      'Die Spaltenbreiten ergeben sich aus dem Headerstring
      Private _HeaderString As String
      Private exp As New System.Text.RegularExpressions.Regex("\S+(\s+|$)")
      Private Mts As System.Text.RegularExpressions.MatchCollection
      Private _SB As New System.Text.StringBuilder
      Private _Default As String

      Public Sub New(HeaderString As String, Optional sDefault As String = "Err")
         _HeaderString = HeaderString.Replace(","c, " ")
         Mts = exp.Matches(_HeaderString)
         _Default = sDefault
         WriteHeader()
      End Sub

      Public Sub DebugWrite(ParamArray Args() As Object)
         Debug.WriteLine(GetString(Args))
      End Sub

      Public Sub WriteHeader(ParamArray Args() As Object)
         Debug.WriteLine(_HeaderString)
      End Sub

      Public Function GetString( _
              ParamArray Args() As Object) As String
         With _SB
            .Remove(0, .Length)
            For I As Integer = 0 To Math.Min(Mts.Count - 1, Args.GetUpperBound(0))
               Dim sArg As String = String.Format("{0}", Args(I))
               If sArg = "" Then sArg = _Default
               .Append(sArg.PadRight(Mts(I).Groups(0).Value.Length))
            Next
            Return .ToString
         End With
      End Function
   End Class 'clsTableDebugger

   Public Class TreenodeComparer
      Implements IComparer
      Implements IComparer(Of TreeNode)
      Private _Images As System.Windows.Forms.ImageList.ImageCollection
      Public Shared Sub CreateSort(TV As TreeView, CompareIcons As Boolean)
         TV.TreeViewNodeSorter = New TreenodeComparer(TV, CompareIcons)
      End Sub
      Private Sub New(TV As TreeView, CompareIcons As Boolean)
         If CompareIcons AndAlso TV.ImageList IsNot Nothing Then _Images = TV.ImageList.Images
      End Sub
      Public Function Compare( _
          x As System.Windows.Forms.TreeNode, _
          y As System.Windows.Forms.TreeNode) As Integer _
      Implements System.Collections.Generic.IComparer(Of System.Windows.Forms.TreeNode).Compare
         'gruppierender Vergleich: zunchst ImageIndex, dann Alphabet
         'dummerweise gibt es 2 Mglichkeiten, den ImageIndex festzulegen: per Key oder per Index
         If _Images IsNot Nothing Then
            Compare = Math.Max(x.ImageIndex, _Images.IndexOfKey(x.ImageKey)) _
                .CompareTo(Math.Max(y.ImageIndex, _Images.IndexOfKey(y.ImageKey)))
            If Compare <> 0 Then Exit Function
         End If
         Return String.Compare(x.Text, y.Text)
      End Function
      Public Function Compare(x As Object, y As Object) As Integer _
      Implements System.Collections.IComparer.Compare
         Return Compare(DirectCast(x, TreeNode), DirectCast(y, TreeNode))
      End Function
   End Class 'TreenodeComparer

   Public Class NotUniqueException
      Inherits ArgumentException
      Public Sub New(ItemNamePlural As String, ItemCount As Integer)
         MyBase.New(String.Format("Nicht eindeutig: {0} {1} gefunden.", ItemCount, ItemNamePlural))
      End Sub
      Public Sub New(ItemNamePlural As String, ItemCount As Integer, DidLookIn As String)
         MyBase.New(String.Format( _
            "Nicht eindeutig: {0} {1} in {2} gefunden.", ItemCount, ItemNamePlural, DidLookIn))
      End Sub
   End Class

   Public Structure ValidateResult
      Public HasException As Boolean
      Public Exception As Exception
      Public Sub New(Ex As Exception)
         Me.Exception = Ex
         HasException = True
      End Sub
      Public Shared ReadOnly Empty As ValidateResult
   End Structure

   Public Class clsFormFinder
      'ruft den Delegaten auf, sobald Ctl auf einem Form aufsitzt. Wichtig, wenn Controls whrend
      '! ihrer Initialisierung auf das Form zugreifen mssen, auf dem sie aufsitzen (werden).

      Private WithEvents _ParentControl As Control
      Private _dlgFormFinder As Action(Of Form)

      Private Sub New(Ctl As Control, FormFoundAction As Action(Of Form))
         _dlgFormFinder = FormFoundAction
         'addhandler _ParentControl.ParentChanged,addressof Ctl_ParentChanged
         Ctl_ParentChanged(Ctl, EventArgs.Empty)
      End Sub

      Private Sub Ctl_ParentChanged(sender As Object, e As System.EventArgs) _
              Handles _ParentControl.ParentChanged
         Dim Ctl As Control = DirectCast(sender, Control)
         Do
            _ParentControl = Ctl
            Ctl = Ctl.Parent
         Loop Until Ctl Is Nothing
         If TypeOf _ParentControl Is Form Then
            _dlgFormFinder(DirectCast(_ParentControl, Form))
            _ParentControl = Nothing
         End If
      End Sub

      Public Shared Sub FindForm(Ctl As Control, FormFoundAction As Action(Of Form))
         Dim O As Object = New clsFormFinder(Ctl, FormFoundAction)
      End Sub

   End Class 'clsFormFinder

   Public Class ValueEventArgs
      Public Shared Function Create(Of T)(Value As T) As ValueEventArgs(Of T)
         Return New ValueEventArgs(Of T)(Value)
      End Function
   End Class 'ValueEventArgs

   Public Class ValueEventArgs(Of T)
      Inherits EventArgs
      Public ReadOnly Value As T
      Public Sub New(Value As T)
         Me.Value = Value
      End Sub
   End Class 'ValueEventArgs

End Namespace