﻿Option Compare Text

Imports System.Runtime.CompilerServices
Imports System.Text
Imports System.Collections
Imports System.Text.RegularExpressions
Namespace System

   Public Module StringX

      Private _SB As New System.Text.StringBuilder


      ''' <summary> verkettet alles </summary>
      <Extension()> _
      Public Function [And](Subj As String, ParamArray others() As Object) As String
         Return String.Concat(Subj, String.Concat(others))
      End Function

      ''' <summary> more comfortable than String.Split(separator as String(),count,mode) </summary>
      <Extension()> _
      Public Function Split(Subj As String, separator As String, count As Integer, Optional mode As StringSplitOptions = StringSplitOptions.None) As String()
         Return Subj.Split(New String() {separator}, count, mode)
      End Function

      ''' <summary> more comfortable to use than String.Split(separator as String(),mode) </summary>
      <Extension()> _
      Public Function Split(Subj As String, separator As String, Optional mode As StringSplitOptions = StringSplitOptions.None) As String()
         Return Subj.Split(New String() {separator}, mode)
      End Function

      ''' <summary> verkettet alles Strings, fügt Separator dazwischen ein </summary>
      <Extension()> _
      Public Function Join( _
         Subj As IEnumerable(Of String), _
         Optional Separator As String = Microsoft.VisualBasic.ControlChars.NewLine) As String
         With New System.Text.StringBuilder
            For Each S In Subj
               .Append(S).Append(Separator)
            Next
            If .Length > 0 Then
               .Remove(.Length - Separator.Length, Separator.Length)
            End If
            Return .ToString
         End With
      End Function

      ''' <summary> fügt ein Object-ParamArray als einen Gesamtstring hinzu </summary>
      <Extension()> _
      Public Sub AddX(Subj As ICollection(Of String), ParamArray Items() As Object)
         With New StringBuilder
            Items.ForEach(Function(itm) .Append(If(itm, "##Null##")))
            Subj.Add(.ToString())
         End With
      End Sub

      ''' <returns>den Stringabschnitt ab dem letzten Auftreten von sMatch (ohne sMatch)</returns>
      <Extension()> _
      Public Function LastRightCut(S$, sMatch$) As String
         Dim I% = S.LastIndexOf(sMatch)
         If I < 0 Then Return Nothing
         Return S.Substring(I + sMatch.Length)
      End Function

      ''' <returns>den Stringabschnitt ab dem ersten Auftreten von sMatch (ohne sMatch)</returns>
      <Extension()> _
      Public Function RightCut(S$, sMatch$) As String
         Dim I% = S.IndexOf(sMatch)
         If I < 0 Then Return Nothing
         Return S.Substring(I + sMatch.Length)
      End Function

      Public Function CreateUnescapeRegex(Pattern As String) As Regex
         Return New Regex("(?<=(^|[^\\](\\\\)*))" & Pattern, RegexOptions.Compiled) 'der vorangestellte Pattern schließt escaped Matches ('\X' etc.) aus
      End Function

      <Extension()> _
      Public Function HasValue(Subj As System.String) As Boolean
         Return Not String.IsNullOrEmpty(Subj)
      End Function

      <Extension()> _
      Public Function HasNoValue(Subj As String) As Boolean
         Return String.IsNullOrEmpty(Subj)
      End Function

      ''' <summary>
      ''' gibt den String-Abschnitt links des ersten gefundenen Matches zurück
      ''' </summary>
      <Extension()> _
      Public Function LastLeftCut(Subj As String, Pattern As String) As String
         Dim Index = Subj.LastIndexOf(Pattern)
         If Index >= 0 Then
            Return Subj.Substring(0, Index)
         Else
            Return Nothing
         End If
      End Function

      ''' <summary>
      ''' gibt den String-Abschnitt links des ersten gefundenen Matches zurück
      ''' </summary>
      <Extension()> _
      Public Function LeftCut(Subj As String, Pattern As String) As String
         Dim Index = Subj.IndexOf(Pattern)
         Return If(Index >= 0, Subj.Substring(0, Index), Nothing)
      End Function

      ' ''Nicht löschen! smart and simple!
      '<Extension()> _
      'Public Function ConcatWith(Subj As Object, ParamArray Args() As Object) As String
      '    'Quasi-Umbau von Shared String.Concat() in eine Object-Function
      '    Return Subj.ToString & String.Concat(Args)
      'End Function

      Public Function Concat(ParamArray Args() As Object) As String
         Return Args.GetEnumerator.Concat
      End Function

      <Extension()> _
      Public Function Concat(Enmrt As IEnumerator) As String
         Return _SB.Clear(Enmrt).ToString
      End Function

      <Extension()> _
      Public Function ConcatWith(Subj As Object, ParamArray Args() As Object) As String
         With _SB
            If Not TypeOf Subj Is String AndAlso TypeOf Subj Is IEnumerable Then
               .Clear(DirectCast(Subj, IEnumerable).GetEnumerator)
            Else
               .Clear(Subj)
            End If
            Return .AddEnumeration(Args.GetEnumerator).ToString
         End With
      End Function

      <Extension()> _
      Public Function Between(Delimiter As Char, ParamArray Args() As Object) As String
         Return Args.Join(Delimiter)
      End Function

      <Extension()> _
      Public Function Between(Delimiter As Char, Args As IEnumerable) As String
         Return Args.Join(Delimiter)
      End Function

      <Extension()> _
      Public Function Between(Delimiter As String, ParamArray Args() As Object) As String
         Return Args.Join(Delimiter)
      End Function

      <Extension()> _
      Public Function Between(Delimiter As String, Args As IEnumerable) As String
         Return Args.Join(Delimiter)
      End Function

      <Extension()> _
      Private Function Join(Subj As IEnumerable, Delimiter As String) As String
         _SB.Clear()
         With Subj.GetEnumerator
            If .MoveNext Then
               _SB.Append(If(.Current, "").ToString)
               While .MoveNext
                  _SB.Append(Delimiter).Append(If(.Current, "").ToString)
               End While
            End If
         End With
         Return _SB.ToString
      End Function

      <Extension()> _
      Public Function IntParsed(ByRef Subj As String) As Integer
         Return Integer.Parse(Subj)
      End Function

      <Extension()> _
      Public Function BoolParsed(ByRef Subj As String) As Boolean
         Return Boolean.Parse(Subj)
      End Function

      <Extension()> _
      Public Function CountChars(S As String, Search As Char) As Integer
         CountChars = 0
         Dim I As Integer = S.IndexOf(Search)
         While I >= 0
            CountChars += 1
            I = S.IndexOf(Search, I + 1)
         End While
      End Function

      <Extension()> _
      Public Function CountSubStrings(S As String, Search As String, Optional Comparison As StringComparison = StringComparison.CurrentCulture) As Integer
         CountSubStrings = 0
         Dim I As Integer = S.IndexOf(Search, 0, Comparison)
         While I >= 0
            CountSubStrings += 1
            I = S.IndexOf(Search, I + 1, Comparison)
         End While
      End Function

   End Module
End Namespace
