﻿Imports System.Linq
Imports System.Collections
Imports System.Collections.Generic
Imports System.Data
Imports System.Diagnostics
Imports System.Runtime.CompilerServices
Imports System.ComponentModel
Namespace System.Windows.Forms

   Public Module BindingSourceX

      ''' <summary> returnt die typisierte Datarow an aktueller Position. </summary>
      <Extension()> _
      Public Function At(Of T As DataRow)(ByVal subj As BindingSource) As T
         Return DirectCast(subj.At(subj.Position), T)
      End Function

      ''' <summary>
      ''' returnt die typisierte Datarow am index. Bei ungültigem index Nothing (keine OutOfRange-Exception!)
      ''' </summary>
      <Extension()> _
      Public Function At(Of T As DataRow)(ByVal subj As BindingSource, ByVal index As Integer) As T
         Return DirectCast(subj.At(index), T)
      End Function

      ''' <summary> returnt die untypisierte Datarow an aktueller Position. </summary>
      <Extension()> _
      Public Function At(ByVal subj As BindingSource) As DataRow
         Return subj.At(subj.Position)
      End Function

      ''' <summary>
      ''' returnt die Datarow am index. Bei ungültigem index Nothing (keine OutOfRange-Exception!)
      ''' </summary>
      <Extension()> _
      Public Function At(ByVal subj As BindingSource, ByVal index As Integer) As DataRow
         If index < 0 OrElse index >= subj.Count Then Return Nothing
         Return DirectCast(subj(index), DataRowView).Row
      End Function

      <Extension()> _
      Public Function FindX(ByVal subj As BindingSource, ByVal columnName As String, ByVal Key As Object) As Integer
         'BindingSource.Find funzt nicht bei relateten BindingSourcses
         'FindX = subj.Find(PropertyName, Key)
         For i As Integer = 0 To subj.Count - 1
            If DirectCast(subj(i), DataRowView)(columnName).Equals(Key) Then
               Return i
            End If
         Next
         Return -1
      End Function

      Private Function KeysMatch(ByVal row As DataRow, ByVal cols As DataColumn(), ByVal keys As Object()) As Boolean
         For ii = 0 To cols.Length - 1
            If Not Object.Equals(keys(ii), row(cols(ii))) Then Return False
         Next
         Return True
      End Function

      ''' <summary> stellt den Datensatz mit den angegebenen Primärschlüssel-Werten ein </summary>
      <Extension()> _
      Public Sub MoveTo(ByVal subj As BindingSource, ByVal primKeys() As Object)
         If subj.Count = 0 Then Return
         Dim rw = DirectCast(subj(0), DataRowView).Row
         Dim cols = rw.Table.PrimaryKey
         Dim i = 0
         Try
            If KeysMatch(rw, cols, primKeys) Then Return
            For i = 1 To subj.Count - 1
               rw = DirectCast(subj(i), DataRowView).Row
               If KeysMatch(rw, cols, primKeys) Then Return
            Next
         Finally
            subj.Position = i
         End Try
      End Sub

      ''' <summary> stellt den Datensatz mit dem angegebenen Primärschlüssel-Wert ein </summary>
      <Extension()> _
      Public Sub MoveTo(ByVal subj As BindingSource, ByVal primKey As Object)
         If subj.Count = 0 Then Return
         subj.MoveTo(DirectCast(subj(0), DataRowView).Row.Table.PrimaryKey(0).ColumnName, primKey)
      End Sub

      ''' <summary> 
      ''' stellt den ersten Datensatz ein, dessen Wert in der angegebenen spalte mit Key übereinstimmt
      ''' </summary>
      <Extension()> _
      Public Sub MoveTo(ByVal subj As BindingSource, ByVal columnName As String, ByVal Key As Object)
         subj.Position = subj.FindX(columnName, Key)
      End Sub

      ''' <summary> stellt die angegebene DataRow ein </summary>
      <Extension()> _
      Public Sub MoveTo(ByVal subj As BindingSource, ByVal row As DataRow)
         Dim dv = DirectCast(subj.List, DataView)
         For i As Integer = 0 To subj.Count - 1
            If dv(i).Row Is row Then
               subj.Position = i
               Return
            End If
         Next
         subj.Position = -1
      End Sub

      <DebuggerStepThroughAttribute()> _
      <Extension()> _
      Public Function AddNewX(Of T As DataRow)(ByVal subj As BindingSource) As T
         Return DirectCast(DirectCast(subj.AddNew(), DataRowView).Row, T)
      End Function

      ''' <summary> returnt eine typisierte Enumeration aller Datarows </summary>
      <Extension()> _
      Public Function GetRows(Of T As DataRow)(ByVal subj As BindingSource) As IEnumerable(Of T)
         Return subj.Cast(Of DataRowView).Select(Function(drv) DirectCast(drv.Row, T))
      End Function

      <DebuggerStepThroughAttribute()> _
      <Extension()> _
      Public Function RowX(Of T As DataRow)(ByVal subj As DataRowView) As T
         Return DirectCast(subj.Row, T)
      End Function

      <DebuggerStepThroughAttribute()> _
      <Extension()> _
      Public Function AddNewX(Of T)(ByVal subj As IBindingList) As T
         Return DirectCast(subj.AddNew(), T)
      End Function


   End Module
End Namespace
