﻿Imports System.Runtime.CompilerServices

Namespace System.Windows.Forms
   Public Module TreeNodeX

      <Extension()> _
      Public Function ParentNodes(ByVal Nd As TreeNode) As TreeNodeCollection
         Dim Parent = Nd.Parent
         Return If(Parent Is Nothing, Nd.TreeView.Nodes, Parent.Nodes)
      End Function

      <Extension()> _
      Public Function PopAt(ByVal Nodes As TreeNodeCollection, ByVal I As Integer) As TreeNode
         PopAt = Nodes(I)
         Nodes.RemoveAt(I)
      End Function

      <Extension()> _
      Public Sub Delete(ByVal Nd As TreeNode, Optional ByVal Recursive As Boolean = False)
         If Not Recursive Then
            'nur diese Instanz ausschneiden, Children an meinen Parent hängen
            Dim ParentNodes = Nd.ParentNodes
            Dim Nodes = Nd.Nodes
            For I = Nodes.Count - 1 To 0 Step -1
               ParentNodes.Add(Nodes.PopAt(I))
            Next
         End If
         Nd.Remove()
      End Sub

      <Extension()> _
      Public Sub EnsureVisibleX( _
            ByVal Nd As TreeNode, _
            ByVal Align As System.Drawing.StringAlignment, Optional ByVal Forced As Boolean = False)
         If Nd.IsVisible AndAlso Not Forced Then Return
         'Zeigt den sichtbar zu machenden Node Oben / Unten / Mitten im Treeview an.
         Dim TV = Nd.TreeView
         Select Case Align
            Case StringAlignment.Near
               TV.TopNode = Nd

            Case StringAlignment.Far
               TV.TopNode = TV.Nodes(0)
               Nd.EnsureVisible()

            Case StringAlignment.Center
               Nd.EnsureVisibleX(StringAlignment.Far)
               With Nd.Bounds
                  Dim Nd2 As TreeNode = TV.GetNodeAt(1, .Bottom + .Height - TV.Height \ 2)
                  If Nd2 IsNot Nothing Then TV.TopNode = Nd2
               End With

         End Select
      End Sub

      <Extension()> _
      Public Function FullKeyPath(ByVal Nd As TreeNode) As String
         With Nd
            If .Name.Length = 0 Then Throw New Exception(String.Concat( _
                  "FullKeyPath kann nicht erstellt werden, da ein Node gefunden", _
                  "wurde, dessen .Name nicht zugewiesen ist."))
            If .Parent Is Nothing Then Return .Name
            Return String.Concat(.Parent.FullKeyPath, .TreeView.PathSeparator, .Name)
         End With
      End Function

      <Extension()> _
      Public Function AddX( _
         ByVal Nodes As TreeNodeCollection, _
         ByVal TextAndKey As String, _
         Optional ByVal IconKey As String = "") As TreeNode
         AddX = New TreeNode(TextAndKey) With { _
            .Name = TextAndKey, _
            .ImageKey = IconKey, _
            .SelectedImageKey = IconKey}
         Nodes.Add(AddX)
      End Function


      Public Structure ApproachResult
         Public ReadOnly Node As TreeNode
         Public ReadOnly StepsDone As Integer
         Public ReadOnly Segments As IList(Of String)

         Public Sub New( _
               ByVal Node As TreeNode, _
               ByVal Segments As IList(Of String), _
               ByVal StepsDone As Integer)
            Me.Node = Node
            Me.Segments = Segments
            Me.StepsDone = StepsDone
         End Sub

         Public ReadOnly Property IsFullMatch() As Boolean
            Get
               Return Segments.Count = StepsDone
            End Get
         End Property
      End Structure 'ApproachResult

      ''' <summary>
      ''' sucht in Nodes und SubNodes anhand eines Pfades nach dem bezeichneten Node.
      ''' Bei Nichtexistenz des ZielNode den übergeordneten, in den der ZielNode zu erstellen wäre
      ''' </summary>
      ''' <param name="PathSegments">Die Segmente des Pfades</param>
      ''' <returns>ein 
      ''' <seealso cref="ApproachResult"> ApproachResult, 
      ''' das den gefundenen Node enthält, und die Anzahl abgearbeiteter Segmente
      ''' </seealso>
      ''' </returns>
      <Extension()> _
      Public Function Approach(ByVal Nodes As TreeNodeCollection, _
         ByVal PathSegments As IList(Of String)) As ApproachResult
         Dim Nd As TreeNode = Nothing, I As Integer
         For I = 0 To PathSegments.Count - 1
            If Not Nodes.ContainsKey(PathSegments(I)) Then Exit For
            Nd = Nodes(PathSegments(I)) ' Node per String-Key referenzieren.
            Nodes = Nd.Nodes
         Next
         Return New ApproachResult(Nd, PathSegments, I)
      End Function

      <Extension()> _
      Public Function Approach(ByVal TV As TreeView, ByVal NodePath As String) As ApproachResult
         Return TV.Nodes.Approach(Strings.Split(NodePath, TV.PathSeparator))
      End Function

   End Module
End Namespace
