Imports Microsoft.VisualBasic
Imports System
Imports System.Text
Imports System.IO
Imports System.Collections
Imports System.Text.RegularExpressions
Imports System.Xml

Public Class PathEx
   'Objekt zum Manipulieren von Dateinamen - bld, da man die Path-Klasse nicht beerben kann
   Private m_Fullname As String

   Public Sub New(ByVal FullnameArg As String)
      m_Fullname = FullnameArg
   End Sub

   Public Property DirectoryName() As String
      Get
         Return Path.GetDirectoryName(m_Fullname)
      End Get
      Set(ByVal Value As String)
         m_Fullname = Value.TrimEnd("\"c) & "\" & Path.GetFileName(m_Fullname)
      End Set
   End Property

   Public Property FileName() As String
      Get
         Return Path.GetFileName(m_Fullname)
      End Get
      Set(ByVal Value As String)
         m_Fullname = DirectoryName & "\" & Value
      End Set
   End Property

   Public Property Extension() As String
      Get
         Return Path.GetExtension(m_Fullname)
      End Get
      Set(ByVal Value As String)
         Dim s = m_Fullname.LastLeftCut(".")
         m_Fullname = String.Concat(If(s,m_Fullname), _
             ".", _
             Value.TrimStart("."c))
      End Set
   End Property

#Region "trivial not shared stuff"

   Public Property Fullname() As String
      Get
         Return m_Fullname
      End Get
      Set(ByVal Value As String)
         m_Fullname = Value
      End Set
   End Property

   Public Function LeftString(ByVal Length As Integer) As String
      Return Fullname.Substring(0, Length)
   End Function

   Public ReadOnly Property FilenameWithoutExtension() As String
      Get
         Return Path.GetFileNameWithoutExtension(m_Fullname)
      End Get
   End Property

   Public ReadOnly Property FullnameWithoutExtension() As String
      Get
         Return FullnameWithoutExtension(m_Fullname)
      End Get
   End Property

   Public Function RelativePath(ByVal sAbsolutePath As String) As String
      Return RelativePath(sAbsolutePath, DirectoryName)
   End Function

   Public Function ShortenFullname(ByVal VisibleLevels As Integer) As String
      Return ShortenFullname(m_Fullname, VisibleLevels)
   End Function

   Public Function AbsolutePath(ByVal RelativePath$) As String
      Return AbsolutePath(RelativePath, DirectoryName)
   End Function

   Public Function AppendNumber( _
               Optional ByVal nDigits As Integer = 2, _
               Optional ByVal sAppend As String = "") As String
      Return AppendNumber(m_Fullname, nDigits, sAppend)
   End Function

   Public Function FillGapsInNumberedPathes( _
               Optional ByVal nDigits As Integer = 2, _
               Optional ByVal Prompt As Boolean = True, _
               Optional ByVal sAppend As String = "") As Boolean
      Return FillGapsInNumberedPathes(m_Fullname, nDigits, Prompt, sAppend)
   End Function
#End Region 'trivial not shared stuff

#Region "shared stuff"

   Public Shared Function FillGapsInNumberedPathes( _
               ByVal FullName As String, _
               Optional ByVal nDigits As Integer = 2, _
               Optional ByVal Prompt As Boolean = True, _
               Optional ByVal sAppend As String = "") As Boolean
      Return EnumerateNumberedPathes(FullName, True, nDigits, Prompt, sAppend) = ""
   End Function

   Public Shared Function EnumerateNumberedPathes( _
               ByVal FullName As String, _
               ByVal bFillGapsMode As Boolean, _
               Optional ByVal nDigits As Integer = 2, _
               Optional ByVal bMustConfirmFillGaps As Boolean = True, _
               Optional ByVal sAppend As String = "") As String
      'gibt einen mit Orndungsnummer verzierten Pfad zurck
      'falls mit bFillGapsMode=True aufgerufen, fhrt er ggfs. Umnumerierungen der Files durch
      Dim sFormat = StrDup(nDigits - 1, "0"c) & "#"c

      Dim NewFullName As String = FullnameWithoutExtension(FullName) & sAppend
      Dim Ext As String = Path.GetExtension(FullName)
      Dim MaxNum As Integer = 0
      Dim Pathes() As String = Directory.GetFileSystemEntries( _
          Path.GetDirectoryName(FullName), _
          Path.GetFileName(NewFullName) & Strings.StrDup(nDigits, "?"c) & Ext)
      Array.Sort(Pathes)
      For Each sName As String In Pathes
         If NewFullName.Length + nDigits > sName.Length Then Continue For
         Dim sNumb As String = sName.Substring(NewFullName.Length, nDigits)
         If Not IsNumeric(sNumb) Then Continue For
         Dim FileNumb As Integer = Integer.Parse(sNumb)
         Select Case FileNumb
            Case MaxNum
               MaxNum += 1
            Case Is > MaxNum                    'Lcke in Numerierung der durchnumerierten Pfade
               If bFillGapsMode Then   'diesen und die folgenden Pfade ggfs. umbenennen, um die
                  '!                                                        Lcke zu schlieen
                  If bMustConfirmFillGaps Then
                     If ConfirmFillGaps(sName) Then
                        bMustConfirmFillGaps = False  'Besttigung der Umbenennerei gilt fr
                        '!                                 alle folgenden Numerierung-Lcken
                     Else
                        Return ""
                     End If
                  End If
                  File.Move(sName, NewFullName & MaxNum.ToString(sFormat) & Ext)
                  FileNumb = MaxNum
               End If
               MaxNum = FileNumb + 1
         End Select
      Next
      Return NewFullName & MaxNum.ToString(sFormat) & Ext
   End Function

   Private Shared Function ConfirmFillGaps(ByVal sName As String) As Boolean
      With New Text.StringBuilder
         .Append("Gap detected in numbered Pathes before:").Append(vbNewLine)
         .Append(sName).Append(vbNewLine)
         .Append("Shall I fill them by renaming the ""wrong"" named files?")
         Return MsgBox(.ToString, MsgBoxStyle.YesNoCancel) = MsgBoxResult.Yes
      End With
   End Function

   Public Shared Sub PreparePath( _
           ByRef sPath As String, Optional ByVal AddSeparator As Boolean = False)
      sPath = PreparedPath(sPath, AddSeparator)
   End Sub

   Public Shared Function LastSegment(ByVal sPath As String) As String
      With sPath.TrimEnd(Path.DirectorySeparatorChar, " "c)
         Return .Substring(.LastIndexOf(Path.DirectorySeparatorChar) + 1)
      End With
   End Function

   Public Shared Function PreparedPath( _
           ByVal sPath As String, Optional ByVal AddSeparator As Boolean = False) As String
      If AddSeparator Then
         Return sPath.TrimEnd("\"c, " "c) & "\"c
      Else
         Return sPath.TrimEnd("\"c, " "c)
      End If
   End Function


   Public Enum GetCommonPathResult As Integer
      NoPathesGiven
      NoneFound
      GuessFileOrFolder
      Success
   End Enum

   ''' <summary>ReturnValue -1 indicates different pathes with no common segment</summary>
   Public Shared Function GetCommonPathLength( _
         ByVal ParamArray Pathes() As String) As Integer
      Return GetCommonPathLength(DirectCast(Pathes, IEnumerable(Of String)))
   End Function

   ''' <summary>ReturnValue -1 indicates different pathes with no common segment</summary>
   Public Shared Function GetCommonPathLength( _
         ByVal FilesystemItems As IEnumerable(Of String)) As Integer
      With FilesystemItems.GetEnumerator
         If Not .MoveNext() Then Return -1
         Dim sCompare As String = .Current
         Dim CommonPathLength As New Extremum(Of Integer)(sCompare.Length)
         Dim sCurrent As String = ""
         While .MoveNext
            sCurrent = .Current
            With CommonPathLength
               .CheckNew(sCurrent.Length)
               Do Until String.Compare( _
                       sCompare.Substring(0, .MinVal), _
                       sCurrent.Substring(0, .MinVal), ignoreCase:=True) = 0
                  .CheckNew(sCompare.LastIndexOf("\"c, .MinVal - 1, .MinVal))
                  If .MinVal < 0 Then Return -1
               Loop
            End With
         End While
         If sCurrent = "" Then
            If File.Exists(sCompare) Then
               CommonPathLength.CheckNew(Path.GetDirectoryName(sCompare).Length)
            ElseIf Not Directory.Exists(sCompare) Then
               Throw New Exception( _
                  "GetCommonPathLength: Es wurde nur 1 Pfad bergeben. ".And( _
                  "Er ist im Filesystem nicht vorhanden. ", _
                  "Es kann nicht ermittelt werden, ob es sich um ein File oder um ein Directory handelt."))
            End If
         End If
         Return CommonPathLength.MinVal
      End With
   End Function

   Public Shared ReadOnly Property FullnameWithoutExtension(ByVal _Fullname As String) As String
      Get
         Return String.Concat( _
             Path.GetDirectoryName(_Fullname), _
             Path.DirectorySeparatorChar, _
             Path.GetFileNameWithoutExtension(_Fullname))
      End Get
   End Property

   Public Shared Function AbsolutePath(ByVal sRelativePath As String, ByVal RelativeTo As String) As String
      Dim Start As Integer
      While sRelativePath.Substring(Start).StartsWith("..\")
         RelativeTo = Path.GetDirectoryName(RelativeTo)
         Start += 3
      End While
      Return RelativeTo & "\" & sRelativePath.Substring(Start)
   End Function

   Public Shared Function RelativePath(ByVal sAbsolutePath$, ByVal sRoot$) As String
      PreparePath(sRoot)
      PreparePath(sAbsolutePath)
      Dim l=GetCommonPathLength(sRoot, sAbsolutePath)
      If l >= 0 Then
         RelativePath = sAbsolutePath.Substring(l + 1)
         'Falls sAbsolutePath gar kein Subfolder von sRoot ist, die entsprechenden "..\" davorsetzen
         For I As Integer = 1 To sRoot.Substring(l).CountChars("\"c)
            RelativePath = "..\" & RelativePath
         Next
      Else
         Throw New Exception( _
            "kann keinen gemeinsamen Pfad finden, fr\n'".And(sRoot, "'\nund\n'", sAbsolutePath, "'"))
      End If
   End Function

   Public Shared Function ShortenFullname( _
           ByVal Fullname As String, ByVal VisibleLevels As Integer) As String
      'haut die ersten Segmente eines Pfades weg, lt die letzten VisibleLevels Segmente stehen.
      Dim Splitted() As String = Fullname.Split("\"c)
      Dim FirstVisible As Integer = Splitted.Length - VisibleLevels
      Return "...\" & String.Join("\", Splitted, FirstVisible, VisibleLevels)
   End Function

   Public Shared Function AppendNumber( _
           ByVal Fullname As String, _
           Optional ByVal nDigits As Integer = 2, _
           Optional ByVal sAppend As String = "") As String
      Return EnumerateNumberedPathes(Fullname, False, nDigits, False, sAppend)
   End Function

   Public Shared Sub CopyForced( _
         ByVal Src As String, _
         ByVal Dst As String, _
         Optional ByVal Errors As List(Of KeyValuePair(Of String, Exception)) = Nothing)
      'kopieren ohne Gnade, Folder oder Files; bestehendes wird berschrieben, nichtexistente
      '! Folder creiert, Folder rekursiv.
      If File.Exists(Src) Then
         Dim sDest As String = Path.GetDirectoryName(Dst)
         If Not Directory.Exists(sDest) Then Directory.CreateDirectory(sDest)
         CopyFile(Src, Dst, Errors)
      ElseIf Directory.Exists(Src) Then
         If File.Exists(Dst) Then Throw New Exception( _
                 Dst & vbNewLine & "kann nicht erstellt werden, da ein gleichnamiges File existiert")
         CopyFolder(Src, Dst, Errors)
      Else
         Throw New Exception("Pfad nicht gefunden:" & vbNewLine & Src)
      End If
   End Sub

   Public Shared Sub CopyFolder( _
         ByVal sSrc As String, _
         ByVal sDest As String, _
         Optional ByVal Errors As List(Of KeyValuePair(Of String, Exception)) = Nothing)
      'rekursiv alle Files eines Folders herumkopieren
      For Each FullName As String In Directory.GetDirectories(sSrc)
         CopyFolder(FullName, sDest & "\" & Path.GetFileName(FullName), Errors)
      Next
      If Not Directory.Exists(sDest) Then Directory.CreateDirectory(sDest)
      For Each FullName As String In Directory.GetFiles(sSrc)
         CopyFile(FullName, sDest & "\" & Path.GetFileName(FullName), Errors)
      Next
   End Sub

   Public Shared Sub CopyFile( _
         ByVal sSrc As String, _
         ByVal sDest As String, _
         Optional ByVal Errors As List(Of KeyValuePair(Of String, Exception)) = Nothing)
      Try
         File.Copy(sSrc, sDest, overwrite:=True)
      Catch ex As Exception
         If Errors IsNot Nothing Then Errors.Add(New KeyValuePair(Of String, Exception)(sSrc, ex))
      End Try
   End Sub

   Public Shared Function UpperDirectory(ByVal Path As String, ByVal Count As Integer) As String
      If Count < 0 Then
         Throw New ArgumentOutOfRangeException("Count", Count, "darf nicht kleiner 0 werden")
      End If
      For i As Integer = 1 To Count
         Path = Global.System.IO.Path.GetDirectoryName(Path)
      Next
      Return Path
   End Function

#End Region 'shared stuff
End Class 'PathEx